import { containerExists, getContainerShell } from './containers.js'
import { WebSocketServer } from 'ws'

export default (server, sessions) => {
  const wss = new WebSocketServer({ noServer: true })

  server.on('upgrade', async (request, socket, head) => {
    const forbidden = () => {
      socket.write('HTTP/1.1 401 Unauthorized\r\n\r\n');
      socket.destroy();
    }

    const path = request.url.substr(1).split('/')

    if (path.length !== 3 || path[0] !== 'ws') {
      return forbidden()
    }

    const [_, container, sessionId] = path
    const session = sessions[sessionId]

    if (session && session.container !== container) {
      console.log('wrong session')
      return forbidden()
    }

    if (!(await containerExists(container))) {
      console.log('no container')
      return forbidden()
    }

    if (!session) {
      sessions[sessionId] = { container }
    }

    request.session = sessions[sessionId]

    wss.handleUpgrade(request, socket, head, ws => {
      wss.emit('connection', ws, request);
    });
  })

  wss.on('connection', (ws, req) => {
    if (!req.session.term) {
      req.session.term = getContainerShell(req.session.container)
    }

    ws.on('message', message => {
      const decoded = message.toString()

      req.session.term.write(decoded)
    })

    req.session.term.onData(data => {
      ws.send(data)
    })

    req.session.term.onExit(exit => {
      ws.send(`Process terminated with code ${exit.exitCode}`)
      ws.close()
    })
  })
}
